// evmone: Fast Ethereum Virtual Machine implementation
// Copyright 2022 The evmone Authors.
// SPDX-License-Identifier: Apache-2.0
#pragma once

#include "../utils/stdx/utility.hpp"
#include <evmc/evmc.hpp>
#include <optional>

namespace evmone::state
{
/// The precompile identifiers and their corresponding addresses.
enum class PrecompileId : uint8_t
{
    ecrecover = 0x01,
    sha256 = 0x02,
    ripemd160 = 0x03,
    identity = 0x04,
    expmod = 0x05,
    ecadd = 0x06,
    ecmul = 0x07,
    ecpairing = 0x08,
    blake2bf = 0x09,
    point_evaluation = 0x0a,
    bls12_g1add = 0x0b,
    bls12_g1mul = 0x0c,
    bls12_g1msm = 0x0d,
    bls12_g2add = 0x0e,
    bls12_g2mul = 0x0f,
    bls12_g2msm = 0x10,
    bls12_pairing_check = 0x11,
    bls12_map_fp_to_g1 = 0x12,
    bls12_map_fp2_to_g2 = 0x13,

    since_byzantium = expmod,         ///< The first precompile introduced in Byzantium.
    since_istanbul = blake2bf,        ///< The first precompile introduced in Istanbul.
    since_cancun = point_evaluation,  ///< The first precompile introduced in Cancun.
    since_prague = bls12_g1add,       ///< The first precompile introduced in Prague.
    latest = bls12_map_fp2_to_g2      ///< The latest introduced precompile (highest address).
};

/// The total number of known precompiles ids, including 0.
inline constexpr std::size_t NumPrecompiles = stdx::to_underlying(PrecompileId::latest) + 1;

/// Checks if the address @p addr is considered a precompiled contract in the revision @p rev.
bool is_precompile(evmc_revision rev, const evmc::address& addr) noexcept;

/// Executes the message to a precompiled contract (msg.code_address must be a precompile).
evmc::Result call_precompile(evmc_revision rev, const evmc_message& msg) noexcept;
}  // namespace evmone::state
